/* srchfrcs.sql  */
/* Created by Laurence D. Finston (LDF)  Undated.  */

/*

$Id: srchfrcs.sql,v 1.1 2006/10/23 16:06:52 ldf Exp $

* (1) Copyright and License.

This file is part of the IWF Metadata Harvester, a package for metadata harvesting.  
Copyright (C) 2006, 2007 IWF Wissen und Medien gGmbH

The author is Laurence D. Finston.                                                   

The IWF Metadata Harvester is free software; you can redistribute it and/or modify   
it under the terms of the GNU General Public License as published by                 
the Free Software Foundation; either version 2 of the License, or                    
(at your option) any later version.                                                  

The IWF Metadata Harvester is distributed in the hope that it will be useful,        
but WITHOUT ANY WARRANTY; without even the implied warranty of                       
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                             
GNU General Public License for more details.                                         

You should have received a copy of the GNU General Public License                    
along with the IWF Metadata Harvester; if not, write to the Free Software            
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA                   

The IWF Metadata Harvester is available for downloading from the                     
following FTP server:                                                                
ftp://ftp.gwdg.de/pub/gnu2/iwfmdh/

Please send bug reports to lfinsto1@gwdg.de                                         

The author can be contacted at:                                                      

Laurence D. Finston 
Kreuzbergring 41    
D-37075 Goettingen  
Germany             

lfinsto1@gwdg.de 
s246794@stud.uni-goettingen.de                        
*/

/* 
LOG

LDF 2006.10.23.
Renamed this file:
Old name:  `search_for_records.sql'
New name:  `srchfrcs.sql'

ENDLOG
*/

/* * (1)  */

/* This procedure writes `Records.record_id' to `Temp_IDs.temp_id' for all the records that fulfill the
   search criteria.  */

/* LDF 2006.06.23.  
   This version uses only `like' for searching, because `contains' and `freetext'
   can't be used for searching _within_ words.  They can be used for `prefixes', 
   though.  
   TO DO:  Add an argument for the kind of search.  If searching for a whole word, 
   or the search string is the beginning for a word, then `contains' or `freetext'
   can be used.  They will be faster, since they use the full-text index, and 
   `like' doesn't.          
*/

-- drop procedure search_for_records
--create
alter 
procedure search_for_records
@search_str nvarchar(128),
@search_options int,
@contributor_value bit,
@creator_value bit,      
@description_value bit,  
@identifier_value bit,   
@language_value bit,     
@publisher_value bit,    
@rights_value bit,        
@subject_value bit,      
@title_value bit,        
@type_value bit          
as

declare @BEG_OR_WHOLE_WORD int
set @BEG_OR_WHOLE_WORD = 1

declare @WHOLE_WORD_ONLY int      
set @WHOLE_WORD_ONLY = 2

declare @WHOLE_OR_PARTIAL_WORD int
set @WHOLE_OR_PARTIAL_WORD = 4

declare @EXACT_MATCH int
set @EXACT_MATCH = 8


if  not (   @search_options = 1 or @search_options = 2 
         or @search_options = 4 or @search_options = 8)
   set @search_options = 1

if @search_options = @BEG_OR_WHOLE_WORD
   begin
      set @search_str = '"' + @search_str + '*"'
   end

if @search_options = @WHOLE_OR_PARTIAL_WORD
   begin
      set @search_str = '%' + @search_str + '%'
   end

delete temp_ids

-- select @search_options as 'Search Options.'

/* Contributors.  */

if (@contributor_value = 1)
   begin
      select 'Searching in "Contributors"'
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids select distinct R.record_id 
            from records as R, contributors as C, records_contributors as RC
            where C.dc_contributor = @search_str 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RC.record_id 
            and C.contributor_id = RC.contributor_id
          
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids select distinct R.record_id 
            from records as R, contributors as C, records_contributors as RC
            where C.dc_contributor like 
            @search_str 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RC.record_id 
            and C.contributor_id = RC.contributor_id
         end
       else
          begin
            insert temp_ids select distinct R.record_id 
            from records as R, contributors as C, records_contributors as RC
            where contains(C.dc_contributor, @search_str) 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RC.record_id 
            and C.contributor_id = RC.contributor_id
          end
   end
else
   begin
      select 'Not searching in "Contributors"'
   end

/* Creators.  */

if (@creator_value = 1)
   begin
      select 'Searching in "Creators"'
      
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids select distinct R.record_id 
            from records as R, creators as C, records_creators as RC
            where C.dc_creator = @search_str 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RC.record_id 
            and C.creator_id = RC.creator_id
         end
         else  
         if @search_options = @WHOLE_OR_PARTIAL_WORD 
            begin
               insert temp_ids select distinct R.record_id 
               from records as R, creators as C, records_creators as RC
               where C.dc_creator like 
               @search_str 
               and R.record_id not in
               (select TID.temp_id from temp_ids as TID)
               and R.record_id = RC.record_id 
               and C.creator_id = RC.creator_id
            end
         else
          begin
            insert temp_ids select distinct R.record_id 
            from records as R, creators as C, records_creators as RC
            where contains(C.dc_creator, @search_str) 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RC.record_id 
            and C.creator_id = RC.creator_id
          end
   end
else
   begin
      select 'Not searching in "Creators"'
   end



/* Descriptions.  */

--set @search_str = stuff(@search_str, 1, 0, '%')
-- set @search_str = stuff(@search_str, len(@search_str) + 1, 1, '%')
-- select @search_str as 'Search String'

if (@description_value = 1)
   begin
      select 'Searching in "Descriptions"'

      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids
            select distinct R.record_id
            from descriptions as D, records as R
            where 
            D.dc_description = @search_str
            and
            D.record_id = R.record_id
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
         end
      else 
         if @search_options = @WHOLE_OR_PARTIAL_WORD 
            begin
               insert temp_ids
               select distinct R.record_id
               from descriptions as D, records as R
               where 
               D.dc_description like @search_str
               and
               D.record_id = R.record_id
               and R.record_id not in
               (select TID.temp_id from temp_ids as TID)
            end
       else
             begin
                insert temp_ids
                select distinct R.record_id
                from descriptions as D, records as R
                where 
                contains(D.dc_description, @search_str)
                and
                D.record_id = R.record_id
                and R.record_id not in
                (select TID.temp_id from temp_ids as TID)
             end
   end
else
   begin
      select 'Not searching in "Descriptions"'
   end

/* Identifiers.  */

if (@identifier_value = 1)
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, identifiers as I, records_identifiers as RI 
            where I.identifier_id in 
            (select I.identifier_id 
            from identifiers as I
            where  
            dc_identifier = @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RI.record_id 
            and I.identifier_id = RI.identifier_id
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, identifiers as I, records_identifiers as RI 
            where I.identifier_id in 
            (select I.identifier_id 
            from identifiers as I
            where  
            dc_identifier like @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RI.record_id 
            and I.identifier_id = RI.identifier_id
         end
       else
          begin
             insert temp_ids
             select distinct R.record_id 
             from records as R, identifiers as I, records_identifiers as RI 
             where I.identifier_id in 
             (select I.identifier_id 
             from identifiers as I 
             where contains(dc_identifier, @search_str))
             and R.record_id not in (select TID.temp_id from temp_ids as TID) 
             and R.record_id = RI.record_id 
             and I.identifier_id = RI.identifier_id
           end
else
   begin
      select 'Not searching in "Identifiers"'
   end


/* Languages.  */

if (@language_value = 1)
      if @search_options = @EXACT_MATCH
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, languages as L, records_languages as RL 
            where L.language_id in 
            (select L.language_id 
            from languages as L
            where  
            dc_language = @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RL.record_id 
            and L.language_id = RL.language_id
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, languages as L, records_languages as RL 
            where L.language_id in 
            (select L.language_id 
            from languages as L
            where  
            dc_language like @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RL.record_id 
            and L.language_id = RL.language_id
         end
       else
          begin
             insert temp_ids
             select distinct R.record_id 
             from records as R, languages as L, records_languages as RL 
             where L.language_id in 
             (select L.language_id 
             from languages as L 
             where contains(dc_language, @search_str))
             and R.record_id not in (select TID.temp_id from temp_ids as TID) 
             and R.record_id = RL.record_id 
             and L.language_id = RL.language_id
           end
else
   begin
      select 'Not searching in "Languages"'
   end

/* Publishers.  */

if (@publisher_value = 1)
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, publishers as P, records_publishers as RP 
            where P.publisher_id in 
            (select P.publisher_id 
            from publishers as P
            where  
            dc_publisher = @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RP.record_id 
            and P.publisher_id = RP.publisher_id
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, publishers as P, records_publishers as RP 
            where P.publisher_id in 
            (select P.publisher_id 
            from publishers as P
            where  
            dc_publisher like @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RP.record_id 
            and P.publisher_id = RP.publisher_id
         end
       else
          begin
             insert temp_ids
             select distinct R.record_id 
             from records as R, publishers as P, records_publishers as RP 
             where P.publisher_id in 
             (select P.publisher_id 
             from publishers as P 
             where contains(dc_publisher, @search_str))
             and R.record_id not in (select TID.temp_id from temp_ids as TID) 
             and R.record_id = RP.record_id 
             and P.publisher_id = RP.publisher_id
           end
else
   begin
      select 'Not searching in "Publishers"'
   end


/* Rights.  */

if (@rights_value = 1)
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, rights as RT, records_rights as RRT 
            where RT.rights_id in 
            (select RT.rights_id 
            from rights as RT
            where  
            dc_rights = @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RRT.record_id 
            and RT.rights_id = RRT.rights_id
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, rights as RT, records_rights as RRT 
            where RT.rights_id in 
            (select RT.rights_id 
            from rights as RT
            where  
            dc_rights like @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RRT.record_id 
            and RT.rights_id = RRT.rights_id
         end
       else
          begin
             insert temp_ids
             select distinct R.record_id 
             from records as R, rights as RT, records_rights as RRT 
             where RT.rights_id in 
             (select RT.rights_id 
             from rights as RT 
             where contains(dc_rights, @search_str))
             and R.record_id not in (select TID.temp_id from temp_ids as TID) 
             and R.record_id = RRT.record_id 
             and RT.rights_id = RRT.rights_id
           end
else
   begin
      select 'Not searching in "Rights"'
   end

/* Subjects.  */

if (@subject_value = 1)
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, subjects as S, records_subjects as RS 
            where S.subject_id in 
            (select S.subject_id 
            from subjects as S
            where  
            dc_subject = @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RS.record_id 
            and S.subject_id = RS.subject_id
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids
            select distinct R.record_id 
               from records as R, subjects as S, records_subjects as RS 
            where S.subject_id in 
            (select S.subject_id 
            from subjects as S
            where  
            dc_subject like @search_str)
            and R.record_id not in (select TID.temp_id from temp_ids as TID) 
            and R.record_id = RS.record_id 
            and S.subject_id = RS.subject_id
         end
       else
          begin
             insert temp_ids
             select distinct R.record_id 
             from records as R, subjects as S, records_subjects as RS 
             where S.subject_id in 
             (select S.subject_id 
             from subjects as S 
             where contains(dc_subject, @search_str))
             and R.record_id not in (select TID.temp_id from temp_ids as TID) 
             and R.record_id = RS.record_id 
             and S.subject_id = RS.subject_id
           end
else
   begin
      select 'Not searching in "Subjects"'
   end

/* Titles.  */

--set @search_str = stuff(@search_str, 1, 0, '%')
-- set @search_str = stuff(@search_str, len(@search_str) + 1, 1, '%')
-- select @search_str as 'Search String'

if (@title_value = 1)
   begin
      select 'Searching in "Titles"'
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids
            select distinct R.record_id
            from titles as T, records as R
            where 
            T.dc_title = @search_str
            and
            T.record_id = R.record_id
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids
            select distinct R.record_id
            from titles as T, records as R
            where 
            T.dc_title like @search_str
            and
            T.record_id = R.record_id
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
         end
       else
          begin
             insert temp_ids
             select distinct R.record_id
             from titles as T, records as R
             where 
             contains(T.dc_title, @search_str)
             and
             T.record_id = R.record_id
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
          end
   end
else
   begin
      select 'Not searching in "Titles"'
   end

/* Types.  */

if (@type_value = 1)
   begin
      select 'Searching in "Types"'
      if @search_options = @EXACT_MATCH
         begin
            insert temp_ids select distinct R.record_id 
            from records as R, types as T, records_types as RT
            where T.dc_type = @search_str 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RT.record_id 
            and T.type_id = RT.type_id
         end
      else
      if @search_options = @WHOLE_OR_PARTIAL_WORD 
         begin
            insert temp_ids select distinct R.record_id 
            from records as R, types as T, records_types as RT
            where T.dc_type like @search_str 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RT.record_id 
            and T.type_id = RT.type_id
         end
       else
          begin
            insert temp_ids select distinct R.record_id 
            from records as R, types as T, records_types as RT
            where contains(T.dc_type, @search_str) 
            and R.record_id not in
            (select TID.temp_id from temp_ids as TID)
            and R.record_id = RT.record_id 
            and T.type_id = RT.type_id
          end
   end
else
   begin
      select 'Not searching in "Types"'
   end
go

-- 
-- select * from temp_ids

-- 


/*  
exec search_for_records 'feil, robert', 8, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0
select * from temp_ids
-- go

select * from temp_ids

select R.record_id, C.dc_creator
from records as R, Creators as C, Records_Creators as RC
where
R.record_id = 32
and
RC.record_id = R.record_id
and
RC.creator_id = C.creator_id

*/

/*
select * from titles
where
contains(dc_title, '"vor*"')
*/

/* Local Variables: */
/* abbrev-mode:t */
/* End: */


